package k4unl.minecraft.k4lib.lib;

import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.world.IBlockAccess;

import java.util.List;

public class Location {
	private int x;
	private int y;
	private int z;
	private int dimension;

	public Location(int x, int y, int z) {
		this.x = x;
		this.y = y;
		this.z = z;
	}

    public Location(String x, String y, String z) {
        this.x = Integer.parseInt(x);
        this.y = Integer.parseInt(y);
        this.z = Integer.parseInt(z);
    }

    public Location(EntityPlayer player){
        this.x = (int)player.field_70165_t;
        this.y = (int)player.field_70163_u;
        this.z = (int)player.field_70161_v;
    }

	public Location(Location clone) {
		this.x = clone.x;
		this.y = clone.y;
		this.z = clone.z;
	}
	
	public Location(int xCoord, int yCoord, int zCoord, EnumFacing dir) {
		this.x = xCoord + dir.func_82601_c();
		this.y = yCoord + dir.func_96559_d();
		this.z = zCoord + dir.func_96559_d();
	}
	
	public Location(int _x, int _y, int _z, int _dimension){
		x = _x;
		y = _y;
		z = _z;
		dimension = _dimension;
	}
	
	public Location(int[] loc){
		if(loc.length > 2){
			x = loc[0];
			y = loc[1];
			z = loc[2];
			if(loc.length > 3){
				dimension = loc[3];
			}
		}else{
			//Log.error("Trying to load a location with a wrong int array!");
		}
	}


	public Location(int _x, int _y, int _z, EnumFacing d, int offset){
		x = _x + (d.func_82601_c() * offset);
		y = _y + (d.func_96559_d() * offset);
		z = _z + (d.func_82599_e() * offset);
	}
	
	public Location(int _x, int _y, int _z, int _dimension, EnumFacing d, int offset){
		x = _x + (d.func_82601_c() * offset);
		y = _y + (d.func_96559_d() * offset);
		z = _z + (d.func_82599_e() * offset);
		dimension = _dimension;
	}
	
	public Location(Location baseLoc, EnumFacing d, int offset){
		x = baseLoc.getX() + (d.func_82601_c() * offset);
		y = baseLoc.getY() + (d.func_96559_d() * offset);
		z = baseLoc.getZ() + (d.func_82599_e() * offset);
		dimension = baseLoc.dimension;
	}
	
	public Location(int _x, int _y, int _z, int _dimension, EnumFacing d) {
		x = _x + d.func_82601_c();
		y = _y + d.func_96559_d();
		z = _z + d.func_82599_e();
		dimension = _dimension;
	}


	public Location(BlockPos pos){
		if(pos != null){
			this.x = pos.func_177958_n();
			this.y = pos.func_177956_o();
			this.z = pos.func_177952_p();
		}
	}


	public Location(MovingObjectPosition blockLookedAt) {
		if(blockLookedAt != null){
			this.x = blockLookedAt.func_178782_a().func_177958_n();
			this.y = blockLookedAt.func_178782_a().func_177956_o();
			this.z = blockLookedAt.func_178782_a().func_177952_p();
		}
	}

	public boolean equals(Location toTest){
		if(this.x == toTest.x && this.y == toTest.y && this.z == toTest.z){
			return true;
		}
		return false;
	}

	public void setLocation(int x, int y, int z) {
		this.x = x;
		this.y = y;
		this.z = z;
	}
	
	public void setLocation(int[] coords) {
		this.x = coords[0];
		this.y = coords[1];
		this.z = coords[2];
	}
	
	public int getX(){
		return this.x;
	}
	
	public int getY(){
		return this.y;
	}
	
	public int getZ(){
		return this.z;
	}
	
	public void setX(int newX){
		this.x = newX;
	}
	
	public void setY(int newY){
		this.y = newY;
	}
	
	public void setZ(int newZ){
		this.z = newZ;
	}

	public int[] getLocation() {
		int[] ret = new int[3];
		ret[0] = this.x;
		ret[1] = this.y;
		ret[2] = this.z;
		return ret;
	}

    public Location getNewOffset(EnumFacing dir, int offset){
        return new Location(this, dir, offset);
    }

	public int getDifference(Location otherLoc){
		return (int)Math.sqrt(Math.pow(this.x - otherLoc.x, 2) + Math.pow(this.y - otherLoc.y, 2) + Math.pow(this.z - otherLoc.z, 2));
	}
	
	public String printLocation(){
		return "X: " + this.x + " Y: " + this.y + " Z: " + this.z;
	}
	
	public String printCoords(){
		return this.x + ", " + this.y + ", " + this.z;
	}
	
	public boolean compare(int x, int y, int z){
		return (this.x == x && this.y == y && this.z == z);
	}
	
	public void addX(int toAdd){
		x += toAdd;
	}
	public void addY(int toAdd){
		y += toAdd;
	}
	public void addZ(int toAdd){
		z += toAdd;
	}

	public void offset(EnumFacing dir, int offsetInt){
		x += dir.func_82601_c() * offsetInt;
		y += dir.func_96559_d() * offsetInt;
		z += dir.func_82599_e() * offsetInt;
	}

	public int[] getIntArray(){
		return new int[] {x, y, z, dimension};
	}

	public Block getBlock(IBlockAccess iba){
		return iba.func_180495_p(new BlockPos(x, y, z)).func_177230_c();
	}


	public Block getBlock(IBlockAccess iba, EnumFacing dir){
		return iba.func_180495_p(new BlockPos(x + dir.func_82601_c(), y + dir.func_96559_d(), z + dir.func_82599_e())).func_177230_c();
	}
	
	public TileEntity getTE(IBlockAccess iba){
		return iba.func_175625_s(new BlockPos(x, y, z));
	}


	public TileEntity getTE(IBlockAccess iba, EnumFacing dir){
		return iba.func_175625_s(new BlockPos(x + dir.func_82601_c(), y + dir.func_96559_d(), z + dir.func_82599_e()));
	}
	
	public String print() {
		return String.format("D: " + dimension + " X: " + x + " Y: " + y + " Z: " + z);
	}
	
	public int getDimension(){
		return dimension;
	}

    public boolean isInList(List<Location> locationList){
        for(Location loc : locationList){
            if(loc.equals(this)){
                return true;
            }
        }
        return false;
    }
}
